import Vuex from 'vuex'
import APIUtil from '@/util/apiutil'
import WAPITransactionSerice from '@/api-services/transaction.service'
import sharedMutations from 'vuex-shared-mutations'
import i18n from './i18n'
import Vue from 'vue'

Vue.use(Vuex)

function getTaListFromLocalStorage() {
  const ta_list = JSON.parse(window.localStorage.getItem('ta_list'))
  if (ta_list && ta_list.length > 0) {
    for (let i = 0; i < ta_list.length; i++) {
      let chg = false
      if (!('ta_type' in ta_list[i])) {
        chg = true
        ta_list[i].ta_type = 'db_editor_simple'
      }
      if (chg) {
        window.localStorage.setItem('ta_list', JSON.stringify(ta_list))
      }
    }
    return ta_list
  } else {
    return []
  }
}

const NetvsVuex = new Vuex.Store({
  state: {
    refreshHandleGPK: null,
    refreshHandleObjType: null,
    ot_lang_attr_def: {},
    xup_perm: {},
    init_fail: false,
    init_fail_info: null,
    spec_ready: !(new Date().getMonth() === 3 && new Date().getDate() === 1),
    user: null,
    token: JSON.parse(window.localStorage.getItem('token')) || null,
    ta_list: getTaListFromLocalStorage(),
    netdb_axios_config: null,
    done: JSON.parse(window.localStorage.getItem('done')) || [],
    undone: JSON.parse(window.localStorage.getItem('undone')) || [],
    undo_redo_new_mutaion: JSON.parse(window.localStorage.getItem('undo_redo_new_mutation')) || true,
    transaction_result: JSON.parse(window.localStorage.getItem('transaction_result')) || null,
    ta_list_errors: JSON.parse(window.localStorage.getItem('ta_list_errors')) || [],
    executing_transaction: JSON.parse(window.localStorage.getItem('executing_transaction')) || false,
    show_sidebar_left: JSON.parse(window.localStorage.getItem('show_sidebar_left')) || true,
    show_sidebar_right: JSON.parse(window.localStorage.getItem('show_sidebar_right')) || false,
    reload_count: 0,
    db_editor_cache: JSON.parse(window.localStorage.getItem('db_editor_cache')) || {},
    db_editor_data_type_cache: null,
    expert: JSON.parse(window.localStorage.getItem('expert')) || false,
    developer: JSON.parse(window.localStorage.getItem('developer')) || false,
    keep_ta: JSON.parse(window.localStorage.getItem('keep_ta')) || false,
    macfinder_jobs: JSON.parse(window.localStorage.getItem('macfinder_jobs')) || [],
    ta_su_user: null,
    impersonate_user: JSON.parse(window.localStorage.getItem('impersonate_user')) || null,
    locale: JSON.parse(window.localStorage.getItem('locale')) || i18n.fallbackLocale[0],
    sys_alerts: [],
    maint_alerts: [],
    default_login_page: JSON.parse(window.localStorage.getItem('default_login_page')) || '/',
    sysinfo: null,
    sysinfo_mods_by_name: null,
    patch_request_actions: JSON.parse(window.localStorage.getItem('patch_request_actions')) || [],
  },
  mutations: {
    updateImpersonatingUser(state, payload) {
      state.impersonate_user = payload
      state.reload_count++
      window.localStorage.setItem('impersonate_user', JSON.stringify(state.impersonate_user))
    },
    updateDefaultLoginPage(state, payload) {
      state.default_login_page = payload
      window.localStorage.setItem('default_login_page', JSON.stringify(state.default_login_page))
    },
    updateSystemAlerts(state, payload) {
      state.sys_alerts = payload
    },
    updateMaintAlerts(state, payload) {
      state.maint_alerts = payload
    },
    populateInitState(state, payload) {
      state.init_fail = payload.init_fail
      state.init_fail_info = payload.init_fail_info
      state.xup_perm = payload.xup_perm
      state.ot_lang_attr_def = payload.ot_lang_attr_def ? APIUtil.dict_of_lists_by_value_of_array(payload.ot_lang_attr_def, 'object_type_fq_name') : {}
      state.user = payload.user
      state.db_editor_data_type_cache = payload.data_types
      state.netdb_axios_config = payload.netdb_axios_config
      state.sysinfo = payload.sysinfo
      state.sysinfo_mods_by_name = payload.sysinfo_mods_by_name
    },
    newMacfinderJob(state, payload) {
      state.macfinder_jobs.push(payload)
      window.localStorage.setItem('macfinder_jobs', JSON.stringify(state.macfinder_jobs))
    },
    toggle_expert(state) {
      state.expert = !state.expert
      window.localStorage.setItem('expert', JSON.stringify(state.expert))
    },
    toggle_developer(state) {
      state.developer = !state.developer
      window.localStorage.setItem('developer', JSON.stringify(state.developer))
    },
    toggle_keep_ta(state) {
      state.keep_ta = !state.keep_ta
      window.localStorage.setItem('keep_ta', JSON.stringify(state.expert))
    },
    emptyState(state) {
      state.ta_list = null
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
    },
    setUndoRedoNewMutation(state, payload) {
      state.undo_redo_new_mutaion = payload
      window.localStorage.setItem('undo_redo_new_mutation', JSON.stringify(state.undo_redo_new_mutaion))
    },
    popUndoRedoDone(state) {
      state.done.pop()
      window.localStorage.setItem('done', JSON.stringify(state.done))
    },
    popUndoRedoUndone(state) {
      state.undone.pop()
      window.localStorage.setItem('undone', JSON.stringify(state.undone))
    },
    pushUndoRedoDone(state, payload) {
      state.done.push(payload)
      window.localStorage.setItem('done', JSON.stringify(state.done))
    },
    pushUndoRedoUndone(state, payload) {
      state.undone.push(payload)
      window.localStorage.setItem('undone', JSON.stringify(state.undone))
    },
    clearUndoRedoDone(state) {
      state.done = []
      window.localStorage.setItem('done', JSON.stringify(state.done))
    },
    clearUndoRedoUndone(state) {
      state.undone = []
      window.localStorage.setItem('undone', JSON.stringify(state.undone))
    },
    moveTaElements(state, payload) {
      state.ta_list.move(payload.from, payload.to)
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
    },
    login(state, payload) {
      const user = payload.user
      const token = payload.token
      const xup_perm = payload.xup_perm
      state.xup_perm = xup_perm
      state.user = user
      state.token = token
      state.ot_lang_attr_def = payload.ot_lang_attr_def ? APIUtil.dict_of_lists_by_value_of_array(payload.ot_lang_attr_def, 'object_type_fq_name') : {}
      window.localStorage.setItem('token', JSON.stringify(state.token))
      if (state.token !== null) {
        state.netdb_axios_config = {
          headers: {
            Authorization: 'Bearer ' + token.token,
            'Content-Type': 'application/json'
          }
        }
      }
    },
    logout(state) {
      state.user = null
      state.token = null
      state.xup_perm = {}
      if (!state.keep_ta) {
        state.ta_list = []
        window.localStorage.removeItem('ta_list')
      }
      state.undo_redo_new_mutaion = true
      state.undone = []
      state.done = []
      state.transaction_result = null
      state.ta_list_errors = []
      state.executing_transaction = false
      state.show_sidebar_right = false
      state.macfinder_jobs = []
      state.impersonate_user = null
      state.ot_lang_attr_def = {}
      window.localStorage.removeItem('token')
      window.localStorage.removeItem('done')
      window.localStorage.removeItem('undone')
      window.localStorage.removeItem('undo_redo_new_mutation')
      window.localStorage.removeItem('transaction_result')
      window.localStorage.removeItem('ta_list_errors')
      window.localStorage.removeItem('executing_transaction')
      window.localStorage.removeItem('show_sidebar_right')
      window.localStorage.removeItem('db_editor_cache')
      window.localStorage.removeItem('macfinder_jobs')
      window.localStorage.removeItem('patch_request_actions')
      window.localStorage.removeItem('impersonate_user')
      state.netdb_axios_config = {
        headers: {
          Authorization: null,
          'Content-Type': 'application/json'
        }
      }
    },
    removeTransactionElement(state, payload) {
      for (let i = 0; i < state.ta_list.length; i++) {
        if (state.ta_list[i].uuid === payload.uuid) {
          state.ta_list.splice(i, 1)
          break
        }
      }
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
    },
    replaceTransactionList(state, new_list) {
      state.ta_list = new_list
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
    },
    addTransactionElement(state, payload) {
      if (state.ta_list == null) {
        state.ta_list = []
      }
      let update = false
      for (let i = 0; i < state.ta_list.length; i++) {
        if (state.ta_list[i].uuid === payload.old_uuid) {
          update = true
          Vue.set(state.ta_list, i, payload)
          break
        }
      }
      if (!update) {
        state.ta_list.push(payload)
      }
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
    },
    emptyTransactionList(state) {
      state.ta_list = []
      state.ta_list_errors = []
      window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      window.localStorage.setItem('ta_list_errors', JSON.stringify(state.ta_list_errors))
    },
    setNavigationRefreshHandle(state, payload) {
      state.refreshHandleObjType = payload.objType
      state.refreshHandleGPK = payload.gpk
    },
    setTransactionResult(state, result) {
      state.transaction_result = result
      window.localStorage.setItem('transaction_result', JSON.stringify(state.transaction_result))
    },
    copyTransactionsWithErrors(state, payload) {
      if (state.ta_list_errors == null) {
        state.ta_list_errors = []
      }
      state.ta_list_errors = payload
      window.localStorage.setItem('ta_list_errors', JSON.stringify(state.ta_list_errors))
    },
    showSidebarLeft(state, show) {
      state.show_sidebar_left = show
      window.localStorage.setItem('show_sidebar_left', show)
    },
    showSidebarRight(state, show) {
      state.show_sidebar_right = show
      window.localStorage.setItem('show_sidebar_right', show)
    },
    setTransactionBusy(state, busy) {
      state.executing_transaction = busy
      window.localStorage.setItem('executing_transaction', busy)
    },
    removeTransactionResult(state) {
      state.transaction_result = null
      window.localStorage.removeItem('transaction_result')
      state.ta_list_errors = []
      window.localStorage.removeItem('ta_list_errors')
    },
    reloadRouterComp(state) {
      state.reload_count++
    },
    cacheDBEditorData(state, data) {
      state.db_editor_cache[data.object_fq_name] = data
      window.localStorage.setItem('db_editor_cache', JSON.stringify(state.db_editor_cache))
    },
    setTaSuUser(state, data) {
      state.ta_su_user = data
    },
    setSpecReady(state, data) {
      state.spec_ready = data
    },
    updateLocale(state, data) {
      state.locale = data
      i18n.locale = state.locale
      window.localStorage.setItem('locale', JSON.stringify(state.locale))
    },
    addPatchRequestAction(state, data) {
      state.patch_request_actions.push(data)
      window.localStorage.setItem('patch_request_actions', JSON.stringify(state.patch_request_actions))
    },
    removePatchRequestAction(state, index) {
      if (index > -1) {
        state.patch_request_actions.splice(index, 1)
        window.localStorage.setItem('patch_request_actions', JSON.stringify(state.patch_request_actions))
      }
    },
    clearPatchRequestActions(state) {
      state.patch_request_actions = []
      window.localStorage.setItem('patch_request_actions', JSON.stringify(state.patch_request_actions))
    }
  },
  getters: {
    patch_request_actions_count: state => {
      return state.patch_request_actions.length
    }
  },
  actions: {
    async testTransaction({commit, state}) {
      try {
        commit('setTransactionBusy', true)
        const api_list = APIUtil.buildAPITaFromTaObjectArray(state)
        try {
          let res
          if (state.ta_su_user == null) {
            res = await WAPITransactionSerice.executeDry(state, api_list)
          } else {
            res = await WAPITransactionSerice.executeDrySu(state, api_list, state.ta_su_user)
          }
          window.console.debug('Test out:', res)
          commit('setTransactionResult', {
            type: 'test-success'
          })
        } catch (e) {
          window.console.error(e)
          const failed_stmt = APIUtil.getAPIErrorNameFromDBException(e.response.data.exception)
          let pos = null
          if (failed_stmt) {
            for (const ta of state.ta_list) {
              if (failed_stmt.startsWith(ta.uuid)) {
                pos = ta
                break
              }
            }
          }
          // Create a copy of the ta_list in ta_list_errors
          commit('copyTransactionsWithErrors', JSON.parse(JSON.stringify(state.ta_list)))
          commit('setTransactionResult', {
            type: 'error',
            error: e.response.data,
            uuid: (typeof e.response.data === 'object' && 'exception' in e.response.data) && pos ? pos.uuid : null
          })
        }
      } finally {
        commit('setTransactionBusy', false)
        if (state.ta_su_user != null) {
          commit('setTaSuUser', null)
        }
      }
    },
    async executeTransaction({commit, state}) {
      commit('setTransactionBusy', true)

      const api_list = APIUtil.buildAPITaFromTaObjectArray(state)
      try {
        const res = await WAPITransactionSerice.execute(state, api_list)
        commit('setTransactionResult', {
          type: 'success',
          result: res.data,
          ta_list: state.ta_list
        })
        commit('emptyTransactionList')
        commit('clearUndoRedoDone')
        commit('clearUndoRedoUndone')
        commit('setUndoRedoNewMutation', true)
      } catch (e) {
        window.console.debug(e.response)
        const failed_stmt = APIUtil.getAPIErrorNameFromDBException(e.response.data.exception)
        let pos = null
        if (failed_stmt) {
          for (const ta of state.ta_list) {
            if (failed_stmt.startsWith(ta.uuid)) {
              pos = ta
              break
            }
          }
        }
        // Create a copy of the ta_list in ta_list_errors
        commit('copyTransactionsWithErrors', JSON.parse(JSON.stringify(state.ta_list)))
        commit('setTransactionResult', {
          type: 'error',
          error: e.response.data,
          uuid: (typeof e.response.data === 'object' && 'exception' in e.response.data) && pos ? pos.uuid : null
        })
      } finally {
        commit('setTransactionBusy', false)
      }
    }
  },
  plugins: [
    // Synced mutation between tabs
    sharedMutations(
      {
        predicate:
          [
            'pushUndoRedoUndone',
            'popUndoRedoUndone',
            'clearUndoRedoUndone',
            'setUndoRedoNewMutation',
            'clearUndoRedoDone',
            'popUndoRedoDone',
            'emptyState',
            'login',
            'logout',
            'addTransactionElement',
            'removeTransactionElement',
            'emptyTransactionList',
            'removeTransactionResult',
            'copyTransactionsWithErrors',
            'setTransactionBusy',
            'moveTaElements',
            'replaceTransactionList',
            'setTransactionResult',
            'newMacfinderJob',
            'toggle_expert',
            'toggle_developer',
            'toggle_keep_ta',
            'updateLocale',
            'reloadRouterComp',
            'addPatchRequestAction',
            'removePatchRequestAction',
            'clearPatchRequestActions',
            'updateImpersonatingUser',
            'updateDefaultLoginPage',
          ]
      }
    )
  ]
})
// Undo-Redo-watched mutations
const watchedMutations = ['addTransactionElement', 'removeTransactionElement', 'emptyTransactionList', 'moveTaElements', 'replaceTransactionList']

export {
  watchedMutations,
  NetvsVuex
}
