from flask import jsonify, g, current_app, request, abort

from net_suite import db
from net_suite.model import BCD
from net_suite.views import api_login_required, get_db_conn, get_current_mgr
from . import dhcp_leases
from . import dhcp_leases_model


def get_dhcpleaseDB():
    if not hasattr(g, 'dhcpleaseDB'):
        g.dhcpleaseDB = dhcp_leases_model.DhcpLeaseDB(database=current_app.config['MINIDHCPVS_DB'],
                                                      user=current_app.config['MINIDHCPVS_USER'],
                                                      host=current_app.config['MINIDHCPVS_HOST'],
                                                      port=current_app.config['MINIDHCPVS_PORT'],
                                                      password=current_app.config['MINIDHCPVS_PW'])
    return g.dhcpleaseDB


@dhcp_leases.teardown_request
def close_db(error):
    if hasattr(g, 'dhcpleaseDB'):
        g.dhcpleaseDB.connection.close()


@dhcp_leases.route("/bcds")
@api_login_required
def usr_bcds():
    dhcpDB = get_dhcpleaseDB()
    user = request.environ['beaker.session']['login']
    user_bcds = user.get_bcds(db, get_db_conn())
    bcd_counts = dhcpDB.get_active_mac_counts([b.name for b in user_bcds])
    return jsonify(bcd_counts)


@dhcp_leases.route("/bcds/<bcd_name>/active")
@api_login_required
def active_leases(bcd_name):
    dhcpDB = get_dhcpleaseDB()
    user = get_current_mgr() 
    user.update_acd_cache(db=db, connection=get_db_conn())
    bcd = BCD.get_by_name(db, get_db_conn(), bcd_name, user.login_name)
    if bcd is None:
        return abort(404)
    leases = dhcpDB.get_active_macs(bcd_name)
    return jsonify(leases)
